@@ -3,7 +3,7 @@ use 5.008005;
 use strict;
 use warnings;
 
-our $VERSION = "0.05";
+our $VERSION = "0.06";
 
 our $draft = "12";
 
@@ -48,7 +48,7 @@ never become production or even finished.
     | Headers (de)compression |    +    |
     | Stream states           |    +    |
     | Flow control            |    ±    |
-    | Server push             |    ±    |
+    | Server push             |    +    |
     | Connect method          |    -    |
 
 
@@ -59,8 +59,8 @@ never become production or even finished.
     | PRIORITY        |    -    |    -    |
     | RST_STREAM      |    +    |    +    |
     | SETTINGS        |    +    |    +    |
-    | PUSH_PROMISE    |    -    |    +    |
-    | PING            |    -    |    -    |
+    | PUSH_PROMISE    |    +    |    +    |
+    | PING            |    +    |    +    |
     | GOAWAY          |    +    |    +    |
     | WINDOW_UPDATE   |    +    |    +    |
     | CONTINUATION    |    ±    |    +    |
