@@ -9,43 +9,57 @@ use Carp;
 
 sub new {
     my ( $class, %opts ) = @_;
-    my $con;
+    my $self = {
+        con            => undef,
+        input          => '',
+        active_streams => 0
+    };
+
     if ( exists $opts{on_push} ) {
         my $cb = delete $opts{on_push};
         $opts{on_new_peer_stream} = sub {
             my $stream_id = shift;
             my $pp_headers;
+            $self->active_streams(+1);
 
-            $con->stream_cb(
+            $self->{con}->stream_cb(
                 $stream_id,
                 RESERVED,
                 sub {
-                    my $res = $cb->( $con->stream_pp_headers($stream_id) );
+                    my $res =
+                      $cb->( $self->{con}->stream_pp_headers($stream_id) );
                     if ( $res && ref $cb eq 'CODE' ) {
-                        $con->stream_cb(
+                        $self->{con}->stream_cb(
                             $stream_id,
                             CLOSED,
                             sub {
                                 $res->(
-                                    $con->stream_headers($stream_id),
-                                    $con->stream_data($stream_id),
+                                    $self->{con}->stream_headers($stream_id),
+                                    $self->{con}->stream_data($stream_id),
                                 );
+                                $self->active_streams(-1);
                             }
                         );
                     }
                     else {
-                        $con->stream_error( $stream_id, REFUSED_STREAM );
+                        $self->{con}
+                          ->stream_error( $stream_id, REFUSED_STREAM );
+                        $self->active_streams(-1);
                     }
                 }
             );
         };
     }
 
-    $con = Protocol::HTTP2::Connection->new( CLIENT, %opts );
-    bless {
-        con   => $con,
-        input => '',
-    }, $class;
+    $self->{con} = Protocol::HTTP2::Connection->new( CLIENT, %opts );
+    bless $self, $class;
+}
+
+sub active_streams {
+    my $self = shift;
+    my $add = shift || 0;
+    $self->{active_streams} += $add;
+    $self->{con}->finish unless $self->{active_streams} > 0;
 }
 
 my @must = (qw(:authority :method :path :scheme));
@@ -55,6 +69,8 @@ sub request {
     my @miss = grep { !exists $h{$_} } @must;
     croak "Missing fields in request: @miss" if @miss;
 
+    $self->active_streams(+1);
+
     my $con = $self->{con};
 
     my $stream_id = $con->new_stream;
@@ -70,6 +86,11 @@ sub request {
         $con->stream_state( $stream_id, HALF_CLOSED );
     }
     else {
+        if ( !$con->preface ) {
+            $con->enqueue( $con->preface_encode,
+                $con->frame_encode( SETTINGS, 0, 0, {} ) );
+            $con->preface(1);
+        }
 
         $con->send_headers(
             $stream_id,
@@ -89,7 +110,7 @@ sub request {
                 $con->stream_headers($stream_id),
                 $con->stream_data($stream_id),
             );
-            $con->finish();
+            $self->active_streams(-1);
         }
     ) if exists $h{on_done};
 
@@ -121,6 +142,7 @@ sub feed {
         $offset += $len;
         $con->upgrade(0);
         $con->enqueue( $con->preface_encode );
+        $con->preface(1);
     }
     while ( $len = $con->frame_decode( \$self->{input}, $offset ) ) {
         tracer->debug("decoded frame at $offset, length $len\n");
