@@ -46,10 +46,6 @@ sub frame_encode {
 
     my $payload = $encoder{$type}->( $con, \$flags, $stream_id, $data_ref );
 
-    # Sended frame may change state of stream
-    $con->state_machine( 'send', $type, $flags, $stream_id )
-      if $type != SETTINGS && $type != GOAWAY && $stream_id != 0;
-
     pack( 'nC2N', length($payload), $type, $flags, $stream_id ) . $payload;
 }
 
@@ -64,12 +60,23 @@ sub preface_encode {
     PREFACE;
 }
 
+sub frame_header_decode {
+    my ( undef, $buf_ref, $buf_offset ) = @_;
+
+    my ( $length, $type, $flags, $stream_id ) =
+      unpack( 'nC2N', substr( $$buf_ref, $buf_offset, 8 ) );
+
+    $length    &= 0x3FFF;
+    $stream_id &= 0x7FFF_FFFF;
+    return $length, $type, $flags, $stream_id;
+}
+
 sub frame_decode {
     my ( $con, $buf_ref, $buf_offset ) = @_;
     return 0 if length($$buf_ref) - $buf_offset < 8;
 
     my ( $length, $type, $flags, $stream_id ) =
-      unpack( 'nC2N', substr( $$buf_ref, $buf_offset, 8 ) );
+      $con->frame_header_decode( $buf_ref, $buf_offset );
 
     # Unknown type of frame
     if ( !exists $frame_class{$type} ) {
@@ -78,9 +85,6 @@ sub frame_decode {
         return undef;
     }
 
-    $length    &= 0x3FFF;
-    $stream_id &= 0x7FFF_FFFF;
-
     return 0 if length($$buf_ref) - $buf_offset - 8 - $length < 0;
 
     tracer->debug(
